@@ -54,9 +54,11 @@ expand_source_and_mask(CursorPtr cursor, CARD32 *data)
 
     p = data;
     fg = ((cursor->foreRed & 0xff00) << 8) |
-        (cursor->foreGreen & 0xff00) | (cursor->foreGreen >> 8);
+          (cursor->foreGreen & 0xff00) |
+          (cursor->foreBlue >> 8);
     bg = ((cursor->backRed & 0xff00) << 8) |
-        (cursor->backGreen & 0xff00) | (cursor->backGreen >> 8);
+          (cursor->backGreen & 0xff00) |
+          (cursor->backBlue >> 8);
     stride = BitmapBytePad(bits->width);
     for (y = 0; y < bits->height; y++)
         for (x = 0; x < bits->width; x++) {
